/**
 * This interface describes an adapter to load / save databases. It is required for the [SyncDatabase](https://nsc-de.github.io/js-database/classes/_database_.syncdatabase.html) class
 *
 * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
 *
 * @see [SyncDatabase](https://nsc-de.github.io/js-database/classes/_database_.syncdatabase.html) - The user of this interface
 * @see [SyncDatabaseAdapter.save()](https://nsc-de.github.io/js-database/interfaces/_database_.databaseadapter.html#save) - 💾 the save method of the [SyncDatabaseAdapter](https://nsc-de.github.io/js-database/interfaces/_database_.databaseadapter.html)
 * @see [SyncDatabaseAdapter.load()](https://nsc-de.github.io/js-database/interfaces/_database_.databaseadapter.html#load) - 💾 the load method of the [SyncDatabaseAdapter](https://nsc-de.github.io/js-database/interfaces/_database_.databaseadapter.html)
 */
export interface SyncDatabaseAdapter {
    /**
     * 💾 Saves the given data
     *
     * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
     * @param data 💾 the data to save
     *
     * @see [SyncDatabaseAdapter](https://nsc-de.github.io/js-database/interfaces/_database_.syncdatabaseadapter.html) - 👩‍👦 the parent interface
     * @see [SyncDatabaseAdapter.load()](https://nsc-de.github.io/js-database/interfaces/_database_.syncdatabaseadapter.html#load) - 💾 the load function
     */
    save(data: JSObject): void;
    /**
     * 💾 loads data
     *
     * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
     * @return 💾 loads the data
     *
     * @see [DatabaseAdapter](https://nsc-de.github.io/js-database/interfaces/_database_.databaseadapter.html) - 👩‍👦 the parent interface
     * @see [DatabaseAdapter.save()](https://nsc-de.github.io/js-database/interfaces/_database_.databaseadapter.html#save) - 💾 the save function
     */
    load(): JSObject;
}
/**
 * This interface describes an adapter to load / save databases. It is required for the [Database](https://nsc-de.github.io/js-database/classes/_database_.database.html) class
 *
 * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
 *
 * @see [Database](https://nsc-de.github.io/js-database/classes/_database_.database.html) - The user of this interface
 * @see [DatabaseAdapter.save()](https://nsc-de.github.io/js-database/interfaces/_database_.databaseadapter.html#save) - 💾 the save method of the [DatabaseAdapter](https://nsc-de.github.io/js-database/interfaces/_database_.databaseadapter.html)
 * @see [DatabaseAdapter.load()](https://nsc-de.github.io/js-database/interfaces/_database_.databaseadapter.html#load) - 💾 the load method of the [DatabaseAdapter](https://nsc-de.github.io/js-database/interfaces/_database_.databaseadapter.html)
 */
export interface DatabaseAdapter {
    /**
     * 💾 Saves the given data
     *
     * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
     * @param data 💾 the data to save
     * @returns ⌛ Promise (Ready when saved) >> ⛔ void
     *
     * @see [DatabaseAdapter](https://nsc-de.github.io/js-database/interfaces/_database_.databaseadapter.html) - 👩‍👦 the parent interface
     * @see [DatabaseAdapter.load()](https://nsc-de.github.io/js-database/interfaces/_database_.databaseadapter.html#load) - 💾 the load function
     */
    save(data: JSObject): Promise<void>;
    /**
     * 💾 loads the data
     *
     * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
     * @return ⌛ Promise (Ready when data is loaded) >> 💾 the data
     *
     * @see [DatabaseAdapter](https://nsc-de.github.io/js-database/interfaces/_database_.databaseadapter.html) - 👩‍👦 the parent interface
     * @see [DatabaseAdapter.save()](https://nsc-de.github.io/js-database/interfaces/_database_.databaseadapter.html#save) - 💾 the save function
     */
    load(): Promise<JSObject>;
}
/**
 * 🔨 Creates an [DatabaseArray](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html) from a given array
 *
 * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
 * @param val 🏡 the value to create a database-value from
 * @returns 🔨 the [DatabaseArray](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html) representation of the given array
 *
 * @see [DatabaseArray](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html)
 * @see [createDatabaseValue()](https://nsc-de.github.io/js-database/modules/_database_.html#createdatabasevalue)
 */
export declare function createDatabaseValue(val: any[]): DatabaseArray;
/**
 * 🔨 Creates an [DatabaseObject](https://nsc-de.github.io/js-database/classes/_database_.databaseobject.html) from a given object
 *
 * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
 * @param val 🏡 the object to create a [DatabaseObject](https://nsc-de.github.io/js-database/classes/_database_.databaseobject.html) from
 * @returns 🔨 the [DatabaseObject](https://nsc-de.github.io/js-database/classes/_database_.databaseobject.html)-representation of the given object
 *
 * @see [DatabaseObject](https://nsc-de.github.io/js-database/classes/_database_.databaseobject.html)
 * @see [createDatabaseValue()](https://nsc-de.github.io/js-database/modules/_database_.html#createdatabasevalue)
 */
export declare function createDatabaseValue(val: JSObject): DatabaseObject;
/**
 * This function is not really useful. It gives just the input back.
 * It is just implemented that you don't get an error when you give
 * a [DatabaseObject](https://nsc-de.github.io/js-database/classes/_database_.databaseobject.html) as argument.
 *
 * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
 * @deprecated ⛔ Using this function with that argument does nothing and just slows down your process
 * @param val 🏡 the value to create a database-value from
 * @returns ⛔ the same [DatabaseObject](https://nsc-de.github.io/js-database/classes/_database_.databaseobject.html) that is given as input
 *
 * @see [DatabaseObject](https://nsc-de.github.io/js-database/classes/_database_.databaseobject.html)
 * @see [createDatabaseValue()](https://nsc-de.github.io/js-database/modules/_database_.html#createdatabasevalue)
 */
export declare function createDatabaseValue(val: DatabaseObject): DatabaseObject;
/**
 * This function is not really useful. It gives just the input back.
 * It is just implemented that you don't get an error when you give
 * a [DatabaseArray](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html) as argument.
 *
 * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
 * @deprecated ⛔ Using this function with that argument does nothing and just slows down your process
 * @param val 📋 the value to create a database-value from
 * @returns ⛔ the same [DatabaseArray](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html) that is given as input
 *
 * @see [DatabaseArray](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html)
 * @see [createDatabaseValue()](https://nsc-de.github.io/js-database/modules/_database_.html#createdatabasevalue)
 */
export declare function createDatabaseValue(val: DatabaseArray): DatabaseArray;
/**
 * This function is not really useful. It gives just the input back.
 * It is just implemented that you don't get an error when you give
 * a string as argument.
 *
 * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
 * @deprecated ⛔ Using this function with that argument does nothing and just slows down your process
 * @param val 📋 the value to create a database-value from
 * @returns ⛔ the same string that is given as input
 *
 * @see [createDatabaseValue()](https://nsc-de.github.io/js-database/modules/_database_.html#createdatabasevalue)
 */
export declare function createDatabaseValue(val: string): string;
/**
 * This function is not really useful. It gives just the input back.
 * It is just implemented that you don't get an error when you give
 * a number as argument.
 *
 * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
 * @deprecated ⛔ Using this function with that argument does nothing and just slows down your process
 * @param val 1️⃣ the value to create a database-value from
 * @returns ⛔ the same number that is given as input
 *
 * @see [createDatabaseValue()](https://nsc-de.github.io/js-database/modules/_database_.html#createdatabasevalue)
 */
export declare function createDatabaseValue(val: number): number;
/**
 * This function is not really useful. It gives just the input back.
 * It is just implemented that you don't get an error when you give
 * a boolean as argument.
 *
 * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
 * @deprecated ⛔ Using this function with that argument does nothing and just slows down your process
 * @param val ❓ the value to create a database-value from
 * @returns ⛔ the same boolean that is given as input
 *
 * @see [createDatabaseValue()](https://nsc-de.github.io/js-database/modules/_database_.html#createdatabasevalue)
 */
export declare function createDatabaseValue(val: boolean): boolean;
/**
 * This function is not really useful. It gives just the input back.
 * It is just implemented that you don't get an error when you give
 * null as argument.
 *
 * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
 * @deprecated ⛔ Using this function with that argument does nothing and just slows down your process
 * @param val ❓ null
 * @returns ⛔ null
 *
 * @see [createDatabaseValue()](https://nsc-de.github.io/js-database/modules/_database_.html#createdatabasevalue)
 */
export declare function createDatabaseValue(val: null): null;
/**
 * This function is not really useful. It gives just the input back.
 * It is just implemented that you don't get an error when you give
 * undefined as argument.
 *
 * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
 * @deprecated ⛔ Using this function with that argument does nothing and just slows down your process
 * @param val ❓ undefined
 * @returns ⛔ undefined
 *
 * @see [createDatabaseValue()](https://nsc-de.github.io/js-database/modules/_database_.html#createdatabasevalue)
 */
export declare function createDatabaseValue(val: undefined): undefined;
/**
 * Creates a [DatabaseValue](https://nsc-de.github.io/js-database/modules/_database_.html#databasevalue) from a given value
 *
 * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
 * @param val 🏡 the value to create a database-value from
 * @returns the database-value
 *
 * @see [createDatabaseValue()](https://nsc-de.github.io/js-database/modules/_database_.html#createdatabasevalue)
 */
export declare function createDatabaseValue(val: DatabaseInsertable): DatabaseValue;
/**
 * 🔨 Creates a normal array from a given [DatabaseArray](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html)
 *
 * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
 * @param val 📋 the [DatabaseArray](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html) you want to convert back to a normal value
 * @returns 🔨 the simple array
 *
 * @see [DatabaseObject](https://nsc-de.github.io/js-database/classes/_database_.databaseobject.html)
 * @see [getNormalValue()](https://nsc-de.github.io/js-database/modules/_database_.html#getnormalvalue)
 */
export declare function getNormalValue(val: DatabaseArray): any[];
/**
 * 🔨 Creates a normal object from a given [DatabaseObject](https://nsc-de.github.io/js-database/classes/_database_.databaseobject.html)
 *
 * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
 * @param val 🏡 the [DatabaseObject](https://nsc-de.github.io/js-database/classes/_database_.databaseobject.html) you want to convert back to a normal value
 * @returns 🔨 the simple object
 *
 * @see [DatabaseObject](https://nsc-de.github.io/js-database/classes/_database_.databaseobject.html)
 * @see [getNormalValue()](https://nsc-de.github.io/js-database/modules/_database_.html#getnormalvalue)
 */
export declare function getNormalValue(val: DatabaseObject): JSObject;
/**
 * This function is not really useful. It gives just the input back.
 * It is just implemented that you don't get an error when you give
 * a array as argument.
 *
 * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
 * @deprecated ⛔ Using this function with that argument does nothing and just slows down your process
 * @param val 🏡 the array you want to convert back to a normal value
 * @returns ⛔ the same array that was given as input
 *
 * @see [DatabaseObject](https://nsc-de.github.io/js-database/classes/_database_.databaseobject.html)
 * @see [getNormalValue()](https://nsc-de.github.io/js-database/modules/_database_.html#getnormalvalue)
 */
export declare function getNormalValue(val: any[]): any[];
/**
 * This function is not really useful. It gives just the input back.
 * It is just implemented that you don't get an error when you give
 * a object as argument.
 *
 * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
 * @deprecated ⛔ Using this function with that argument does nothing and just slows down your process
 * @param val 🏡 the object you want to convert back to a normal value
 * @returns ⛔ the same object that was given as input
 *
 * @see [DatabaseObject](https://nsc-de.github.io/js-database/classes/_database_.databaseobject.html)
 * @see [getNormalValue()](https://nsc-de.github.io/js-database/modules/_database_.html#getnormalvalue)
 */
export declare function getNormalValue(val: JSObject): JSObject;
/**
 * This function is not really useful. It gives just the input back.
 * It is just implemented that you don't get an error when you give
 * a string as argument.
 *
 * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
 * @deprecated ⛔ Using this function with that argument does nothing and just slows down your process
 * @param val 📋 the string you want to convert back to a normal value
 * @returns ⛔ the same string that was given as input
 *
 * @see [getNormalValue()](https://nsc-de.github.io/js-database/modules/_database_.html#getnormalvalue)
 */
export declare function getNormalValue(val: string): string;
/**
 * This function is not really useful. It gives just the input back.
 * It is just implemented that you don't get an error when you give
 * a number as argument.
 *
 * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
 * @deprecated ⛔ Using this function with that argument does nothing and just slows down your process
 * @param val 1️⃣ the number you want to convert back to a normal value
 * @returns ⛔ the same number that was given as input
 *
 * @see [getNormalValue()](https://nsc-de.github.io/js-database/modules/_database_.html#getnormalvalue)
 */
export declare function getNormalValue(val: number): number;
/**
 * This function is not really useful. It gives just the input back.
 * It is just implemented that you don't get an error when you give
 * a boolean as argument.
 *
 * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
 * @deprecated ⛔ Using this function with that argument does nothing and just slows down your process
 * @param val ❓ the boolean you want to convert back to a normal value
 * @returns ⛔ the same boolean that was given as input
 *
 * @see [getNormalValue()](https://nsc-de.github.io/js-database/modules/_database_.html#getnormalvalue)
 */
export declare function getNormalValue(val: boolean): boolean;
/**
 * This function is not really useful. It gives just the input back.
 * It is just implemented that you don't get an error when you give
 * null as argument.
 *
 * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
 * @deprecated ⛔ Using this function with that argument does nothing and just slows down your process
 * @param val ❓ null
 * @returns ⛔ null
 *
 * @see [getNormalValue()](https://nsc-de.github.io/js-database/modules/_database_.html#getnormalvalue)
 */
export declare function getNormalValue(val: null): null;
/**
 * This function is not really useful. It gives just the input back.
 * It is just implemented that you don't get an error when you give
 * undefined as argument.
 *
 * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
 * @deprecated ⛔ Using this function with that argument does nothing and just slows down your process
 * @param val ❓ undefined
 * @returns ⛔ undefined
 *
 * @see [getNormalValue()](https://nsc-de.github.io/js-database/modules/_database_.html#getnormalvalue)
 */
export declare function getNormalValue(val: undefined): undefined;
/**
 * creates the 'normal' value from [DatabaseValue](https://nsc-de.github.io/js-database/modules/_database_.html#databasevalue)
 *
 * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
 * @param val 🏡 the value you want to convert back to a normal value
 * @returns the 'normal' value
 *
 * @see [getNormalValue()](https://nsc-de.github.io/js-database/modules/_database_.html#getnormalvalue)
 */
export declare function getNormalValue(val: DatabaseInsertable): DatabaseValueAble;
/**
 * A [DatabaseArray](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html) is the database-representation of an array.
 *
 * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
 *
 * @see [DatabaseArray](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html).[data](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html#data) - the data of the array
 * @see [DatabaseArray](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html).[length](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html#length) - the array length
 * @see [DatabaseArray](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html).[constructor()](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html#constructor) - the constructor
 * @see [DatabaseArray](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html).[set()](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html#set) - sets a value from the [DatabaseArray](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html)
 * @see [DatabaseArray](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html).[setDefault()](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html#setDefault) - sets a default value from the [DatabaseArray](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html)
 * @see [DatabaseArray](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html).[get()](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html#get) - gets a value from the [DatabaseArray](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html)
 * @see [DatabaseArray](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html).[getNormal()](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html#getNormal) - gets a value from the [DatabaseArray](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html) and normalizes it
 * @see [DatabaseArray](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html).[update()](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html#update) - updates a value from the [DatabaseArray](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html)
 * @see [DatabaseArray](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html).[contains()](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html#contains) - checks if the [DatabaseArray](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html) contains a value
 * @see [DatabaseArray](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html).[push()](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html#push) - push a value into the [DatabaseArray](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html)
 */
export declare class DatabaseArray {
    _data: DatabaseValueAble[];
    /**
     * The data of the [DatabaseArray](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html)
     *
     * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
     *
     * @see [DatabaseArray](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html) - 👩‍👦 the parent class
     * @see [DatabaseArray](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html).[data](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html#_data) - the storage for the data
     */
    set data(data: DatabaseValueAble[]);
    get data(): DatabaseValueAble[];
    /**
     * Returns the length of the array
     *
     * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
     *
     * @see [DatabaseArray](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html) - 👩‍👦 the parent class
     * @see [DatabaseArray](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html).[data](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html#data) - the data of the array
     */
    get length(): number;
    /**
     * Constructor for [DatabaseArray](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html)
     *
     * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
     * @param data the data for the database-array
     *
     * @see [DatabaseArray](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html) - 👩‍👦 the parent class
     * @see [DatabaseArray](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html).[data](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html#data) - the argument you give to the constructor
     */
    constructor(data?: Array<any>);
    /**
     * Sets a value from the [DatabaseArray](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html).
     *
     * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
     * @param key the path to apply the value to
     * @param value the value to apply to the given path
     * @returns the DatabaseArray itself, so you can chain operations like that
     *
     * @see [DatabaseArray](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html) - 👩‍👦 the parent class
     */
    set(key: number | string | Array<string | number>, value: DatabaseInsertable): this;
    /**
     * Sets a default value from the [DatabaseArray](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html).
     *
     * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
     * @param key the path to apply the value to
     * @param value the value to apply to the given path
     * @returns the [DatabaseArray](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html) itself, so you can chain operations like that
     *
     * @see [DatabaseArray](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html) - 👩‍👦 the parent class
     */
    setDefault(key: number | string | Array<string | number>, value: DatabaseInsertable): this;
    /**
     * Gets a value from the [DatabaseArray](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html).
     *
     * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
     * @param key the path to get
     * @returns the value of the key
     *
     * @see [DatabaseArray](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html) - 👩‍👦 the parent class
     */
    get(key: number | string | Array<string | number>): DatabaseValue;
    /**
     * Gets a value from the [DatabaseArray](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html) normalizes it
     *
     * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
     * @param key the path to get
     * @returns the value of the key
     *
     * @see [DatabaseArray](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html) - 👩‍👦 the parent class
     */
    getNormal(key: number | string | Array<string | number>): DatabaseValueAble;
    /**
     * Updates a value from the [DatabaseArray](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html).
     *
     * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
     * @param key the path to update
     * @param update the function to update the value
     * @returns the [DatabaseArray](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html) itself, so you can chain operations like that
     *
     * @see [DatabaseArray](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html) - 👩‍👦 the parent class
     */
    update(key: number | string | Array<string | number>, update: (e: DatabaseValue) => DatabaseInsertable): this;
    /**
     * Checks if the [DatabaseArray](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html) contains a value
     *
     * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
     * @param key the path to check
     * @returns does the array contains the value
     *
     * @see [DatabaseArray](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html) - 👩‍👦 the parent class
     */
    contains(key: number | string | Array<string | number>): boolean;
    /**
     * Pushes a value into the [DatabaseArray](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html)
     *
     * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
     * @param value the value to push into the array
     * @returns the [DatabaseArray](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html) itself, so you can chain operations like that
     *
     * @see [DatabaseArray](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html) - 👩‍👦 the parent class
     */
    push(...values: DatabaseInsertable[]): this;
}
/**
 * A [DatabaseObject](https://nsc-de.github.io/js-database/classes/_database_.databaseobject.html) is the database-representation of an object.
 *
 * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
 *
 * @see [DatabaseObject](https://nsc-de.github.io/js-database/classes/_database_.databaseobject.html).[data](https://nsc-de.github.io/js-database/classes/_database_.databaseobject.html#data) - the data of the array
 * @see [DatabaseObject](https://nsc-de.github.io/js-database/classes/_database_.databaseobject.html).[length](https://nsc-de.github.io/js-database/classes/_database_.databaseobject.html#length) - the array length
 * @see [DatabaseObject](https://nsc-de.github.io/js-database/classes/_database_.databaseobject.html).[constructor()](https://nsc-de.github.io/js-database/classes/_database_.databaseobject.html#constructor) - the constructor
 * @see [DatabaseObject](https://nsc-de.github.io/js-database/classes/_database_.databaseobject.html).[set()](https://nsc-de.github.io/js-database/classes/_database_.databaseobject.html#set) - sets a value from the [DatabaseObject](https://nsc-de.github.io/js-database/classes/_database_.databaseobject.html)
 * @see [DatabaseObject](https://nsc-de.github.io/js-database/classes/_database_.databaseobject.html).[setDefault()](https://nsc-de.github.io/js-database/classes/_database_.databaseobject.html#setDefault) - sets a default value from the [DatabaseObject](https://nsc-de.github.io/js-database/classes/_database_.databaseobject.html)
 * @see [DatabaseObject](https://nsc-de.github.io/js-database/classes/_database_.databaseobject.html).[setDefaults()](https://nsc-de.github.io/js-database/classes/_database_.databaseobject.html#setDefaults) - sets defaults for the [DatabaseObject](https://nsc-de.github.io/js-database/classes/_database_.databaseobject.html)
 * @see [DatabaseObject](https://nsc-de.github.io/js-database/classes/_database_.databaseobject.html).[get()](https://nsc-de.github.io/js-database/classes/_database_.databaseobject.html#get) - gets a value from the [DatabaseObject](https://nsc-de.github.io/js-database/classes/_database_.databaseobject.html)
 * @see [DatabaseObject](https://nsc-de.github.io/js-database/classes/_database_.databaseobject.html).[getNormal()](https://nsc-de.github.io/js-database/classes/_database_.databaseobject.html#getNormal) - gets a value from the [DatabaseObject](https://nsc-de.github.io/js-database/classes/_database_.databaseobject.html) and normalizes it
 * @see [DatabaseObject](https://nsc-de.github.io/js-database/classes/_database_.databaseobject.html).[update()](https://nsc-de.github.io/js-database/classes/_database_.databaseobject.html#update) - updates a value from the [DatabaseObject](https://nsc-de.github.io/js-database/classes/_database_.databaseobject.html)
 * @see [DatabaseObject](https://nsc-de.github.io/js-database/classes/_database_.databaseobject.html).[contains()](https://nsc-de.github.io/js-database/classes/_database_.databaseobject.html#contains) - checks if the [DatabaseObject](https://nsc-de.github.io/js-database/classes/_database_.databaseobject.html) contains a value
 * @see [DatabaseObject](https://nsc-de.github.io/js-database/classes/_database_.databaseobject.html).[generateId()](https://nsc-de.github.io/js-database/classes/_database_.databaseobject.html#contains) - Generates an id using a namespace
 */
export declare class DatabaseObject {
    private _data;
    /**
     * The data of the DatabaseObject
     *
     * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
     *
     * @see [DatabaseObject](https://nsc-de.github.io/js-database/classes/_database_.databaseobject.html) - 👩‍👦 the parent class
     */
    set data(data: JSObject<DatabaseValueAble>);
    get data(): JSObject<DatabaseValueAble>;
    /**
     * Returns the length of the object
     *
     * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
     *
     * @see [DatabaseArray](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html) - 👩‍👦 the parent class
     * @see [DatabaseArray](https://nsc-de.github.io/js-database/classes/_database_.databasearray.html).data - the data of the array
     */
    get length(): number;
    /**
     * Constructor for DatabaseObject
     *
     * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
     * @param data the data for the database-array
     *
     * @see [DatabaseObject](https://nsc-de.github.io/js-database/classes/_database_.databaseobject.html) - 👩‍👦 the parent class
     * @see [DatabaseObject](https://nsc-de.github.io/js-database/classes/_database_.databaseobject.html).data - the argument you give to the constructor
     */
    constructor(data?: JSObject<DatabaseValueAble>);
    /**
     * Sets a value from the DatabaseObject.
     *
     * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
     * @param key the path to apply the value to
     * @param value the value to apply to the given path
     * @returns the DatabaseObject itself, so you can chain operations like that
     *
     * @see [DatabaseObject](https://nsc-de.github.io/js-database/classes/_database_.databaseobject.html) - 👩‍👦 the parent class
     */
    set(key: string | Array<string | number>, value: DatabaseInsertable): this;
    /**
     * Sets a default value from the [DatabaseObject](https://nsc-de.github.io/js-database/classes/_database_.databaseobject.html).
     *
     * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
     * @param key the path to apply the value to
     * @param value the value to apply to the given path
     * @returns the [DatabaseObject](https://nsc-de.github.io/js-database/classes/_database_.databaseobject.html) itself, so you can chain operations like that
     *
     * @see [DatabaseObject](https://nsc-de.github.io/js-database/classes/_database_.databaseobject.html) - 👩‍👦 the parent class
     */
    setDefault(key: string | Array<string | number>, value: DatabaseInsertable): this;
    /**
     * Sets default values for the DatabaseObject.
     *
     * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
     * @param defaults an object containing all the default values
     * @returns the [DatabaseObject](https://nsc-de.github.io/js-database/classes/_database_.databaseobject.html) itself, so you can chain operations like that
     *
     * @see [DatabaseObject](https://nsc-de.github.io/js-database/classes/_database_.databaseobject.html) - 👩‍👦 the parent class
     */
    setDefaults(defaults: JSObject<DatabaseInsertable>): this;
    /**
     * Gets a value from the DatabaseObject.
     *
     * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
     * @param key the path to get
     * @returns the value at the position
     *
     * @see [DatabaseObject](https://nsc-de.github.io/js-database/classes/_database_.databaseobject.html) - 👩‍👦 the parent class
     */
    get(key: string | Array<string | number>): DatabaseValue;
    /**
     * Gets a value from the [DatabaseObject](https://nsc-de.github.io/js-database/classes/_database_.databaseobject.html) and normalizes it
     *
     * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
     * @param key the path to get
     * @returns the normalized value at the position
     *
     * @see [DatabaseObject](https://nsc-de.github.io/js-database/classes/_database_.databaseobject.html) - 👩‍👦 the parent class
     */
    getNormal(key: string | Array<string | number>): DatabaseValueAble;
    /**
     * Updates a value from the [DatabaseObject](https://nsc-de.github.io/js-database/classes/_database_.databaseobject.html).
     *
     * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
     * @param key the path to update
     * @param update the function to update the value
     * @returns the [DatabaseObject](https://nsc-de.github.io/js-database/classes/_database_.databaseobject.html) itself, so you can chain operations like that
     *
     * @see [DatabaseObject](https://nsc-de.github.io/js-database/classes/_database_.databaseobject.html) - 👩‍👦 the parent class
     */
    update(key: string | Array<string | number>, update: (e: DatabaseValue) => DatabaseInsertable): this;
    /**
     * Checks if the [DatabaseObject](https://nsc-de.github.io/js-database/classes/_database_.databaseobject.html) contains a value
     *
     * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
     * @param key the path to apply the value to
     * @returns the value at the position of the position
     *
     * @see [DatabaseObject](https://nsc-de.github.io/js-database/classes/_database_.databaseobject.html) - 👩‍👦 the parent class
     */
    contains(key: string | Array<string | number>): boolean;
    /**
     * Generates an using a namespace
     *
     * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
     * @param name the name to generate a namespace from
     * @returns the generated id
     *
     * @see [DatabaseObject](https://nsc-de.github.io/js-database/classes/_database_.databaseobject.html) - 👩‍👦 the parent class
     */
    generateId(name: string): number;
}
/**
 * Database class is basically a [DatabaseObject](https://nsc-de.github.io/js-database/classes/_database_.databaseobject.html) with save and load functionality
 *
 * @see [SyncDatabase](https://nsc-de.github.io/js-database/classes/_database_.syncdatabase.html) - A database that works sync instead of using promises
 * @see [Database](https://nsc-de.github.io/js-database/classes/_database_.database.html).[constructor()](https://nsc-de.github.io/js-database/classes/_database_.database.html#constructor) - the constructor
 * @see [Database](https://nsc-de.github.io/js-database/classes/_database_.database.html).[saveData()](https://nsc-de.github.io/js-database/classes/_database_.database.html#savedata) - saves the data to the [DatabaseAdapter](https://nsc-de.github.io/js-database/interfaces/_database_.databaseadapter.html)
 * @see [Database](https://nsc-de.github.io/js-database/classes/_database_.database.html).[reloadData()](https://nsc-de.github.io/js-database/classes/_database_.database.html#reloaddata) - loads the data from the [DatabaseAdapter](https://nsc-de.github.io/js-database/interfaces/_database_.databaseadapter.html)
 *
 * @see [Database](https://nsc-de.github.io/js-database/classes/_database_.database.html).[data](https://nsc-de.github.io/js-database/classes/_database_.database.html#data) - the data of the array
 * @see [Database](https://nsc-de.github.io/js-database/classes/_database_.database.html).[length](https://nsc-de.github.io/js-database/classes/_database_.database.html#length) - the array length
 * @see [Database](https://nsc-de.github.io/js-database/classes/_database_.database.html).[set()](https://nsc-de.github.io/js-database/classes/_database_.database.html#set) - sets a value from the [Database](https://nsc-de.github.io/js-database/classes/_database_.database.html)
 * @see [Database](https://nsc-de.github.io/js-database/classes/_database_.database.html).[setDefault()](https://nsc-de.github.io/js-database/classes/_database_.database.html#setDefault) - sets a default value from the [Database](https://nsc-de.github.io/js-database/classes/_database_.database.html)
 * @see [Database](https://nsc-de.github.io/js-database/classes/_database_.database.html).[setDefaults()](https://nsc-de.github.io/js-database/classes/_database_.database.html#setDefaults) - sets defaults for the [Database](https://nsc-de.github.io/js-database/classes/_database_.database.html)
 * @see [Database](https://nsc-de.github.io/js-database/classes/_database_.database.html).[get()](https://nsc-de.github.io/js-database/classes/_database_.database.html#get) - gets a value from the [Database](https://nsc-de.github.io/js-database/classes/_database_.database.html)
 * @see [Database](https://nsc-de.github.io/js-database/classes/_database_.database.html).[getNormal()](https://nsc-de.github.io/js-database/classes/_database_.database.html#getNormal) - gets a value from the [Database](https://nsc-de.github.io/js-database/classes/_database_.database.html) and normalizes it
 * @see [Database](https://nsc-de.github.io/js-database/classes/_database_.database.html).[update()](https://nsc-de.github.io/js-database/classes/_database_.database.html#update) - updates a value from the [Database](https://nsc-de.github.io/js-database/classes/_database_.database.html)
 * @see [Database](https://nsc-de.github.io/js-database/classes/_database_.database.html).[contains()](https://nsc-de.github.io/js-database/classes/_database_.database.html#contains) - checks if the [Database](https://nsc-de.github.io/js-database/classes/_database_.database.html) contains a value
 * @see [Database](https://nsc-de.github.io/js-database/classes/_database_.database.html).[generateId()](https://nsc-de.github.io/js-database/classes/_database_.database.html#contains) - checks if the [Database](https://nsc-de.github.io/js-database/classes/_database_.database.html) - Generates an id using a namespace
 */
export declare class Database extends DatabaseObject {
    adapter: DatabaseAdapter;
    /**
     * 🔨 Constructor for Database class
     *
     * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
     * @param adapter the adapter to get the data from / save the data to
     * @param data the data to create the database with
     *
     * @see [Database](https://nsc-de.github.io/js-database/classes/_database_.database.html) - 👩‍👦 the parent class
     */
    constructor(adapter: DatabaseAdapter, data: JSObject<DatabaseValueAble>);
    /**
     * 💾 Save the data from the database
     *
     * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
     * @returns ⌛ Promise (Ready when saved) >> the database itself, so you can chain operations
     *
     * @see [Database](https://nsc-de.github.io/js-database/classes/_database_.database.html) - 👩‍👦 the parent class
     */
    saveData(): Promise<this>;
    /**
     * 🔄 Reload the data from the database
     *
     * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
     * @returns ⌛ Promise (Ready when loaded) >> the database itself, so you can chain operations
     *
     * @see [Database](https://nsc-de.github.io/js-database/classes/_database_.database.html) - 👩‍👦 the parent class
     */
    reloadData(): Promise<this>;
}
/**
 * SyncDatabase class is basically a [DatabaseObject](https://nsc-de.github.io/js-database/classes/_database_.databaseobject.html) with save and load functionality.
 * It is written synchronously (without using promises)
 *
 * @see [Database](https://nsc-de.github.io/js-database/classes/_database_.syncdatabase.html) - A database that works using promises for better performance
 * @see [SyncDatabase](https://nsc-de.github.io/js-database/classes/_database_.syncdatabase.html).[constructor()](https://nsc-de.github.io/js-database/classes/_database_.syncdatabase.html#constructor) - the constructor
 * @see [SyncDatabase](https://nsc-de.github.io/js-database/classes/_database_.syncdatabase.html).[saveData()](https://nsc-de.github.io/js-database/classes/_database_.syncdatabase.html#savedata) - saves the data to the [DatabaseAdapter](https://nsc-de.github.io/js-database/interfaces/_database_.syncdatabaseadapter.html)
 * @see [SyncDatabase](https://nsc-de.github.io/js-database/classes/_database_.syncdatabase.html).[reloadData()](https://nsc-de.github.io/js-database/classes/_database_.syncdatabase.html#reloaddata) - loads the data from the [DatabaseAdapter](https://nsc-de.github.io/js-database/interfaces/_database_.syncdatabaseadapter.html)
 *
 * @see [SyncDatabase](https://nsc-de.github.io/js-database/classes/_database_.syncdatabase.html).[data](https://nsc-de.github.io/js-database/classes/_database_.syncdatabase.html#data) - the data of the array
 * @see [SyncDatabase](https://nsc-de.github.io/js-database/classes/_database_.syncdatabase.html).[length](https://nsc-de.github.io/js-database/classes/_database_.syncdatabase.html#length) - the array length
 * @see [SyncDatabase](https://nsc-de.github.io/js-database/classes/_database_.syncdatabase.html).[set()](https://nsc-de.github.io/js-database/classes/_database_.syncdatabase.html#set) - sets a value from the [SyncDatabase](https://nsc-de.github.io/js-database/classes/_database_.syncdatabase.html)
 * @see [SyncDatabase](https://nsc-de.github.io/js-database/classes/_database_.syncdatabase.html).[setDefault()](https://nsc-de.github.io/js-database/classes/_database_.syncdatabase.html#setDefault) - sets a default value from the [SyncDatabase](https://nsc-de.github.io/js-database/classes/_database_.syncdatabase.html)
 * @see [SyncDatabase](https://nsc-de.github.io/js-database/classes/_database_.syncdatabase.html).[setDefaults()](https://nsc-de.github.io/js-database/classes/_database_.syncdatabase.html#setDefaults) - sets defaults for the [SyncDatabase](https://nsc-de.github.io/js-database/classes/_database_.syncdatabase.html)
 * @see [SyncDatabase](https://nsc-de.github.io/js-database/classes/_database_.syncdatabase.html).[get()](https://nsc-de.github.io/js-database/classes/_database_.syncdatabase.html#get) - gets a value from the [SyncDatabase](https://nsc-de.github.io/js-database/classes/_database_.syncdatabase.html)
 * @see [SyncDatabase](https://nsc-de.github.io/js-database/classes/_database_.syncdatabase.html).[getNormal()](https://nsc-de.github.io/js-database/classes/_database_.syncdatabase.html#getNormal) - gets a value from the [SyncDatabase](https://nsc-de.github.io/js-database/classes/_database_.syncdatabase.html) and normalizes it
 * @see [SyncDatabase](https://nsc-de.github.io/js-database/classes/_database_.syncdatabase.html).[update()](https://nsc-de.github.io/js-database/classes/_database_.syncdatabase.html#update) - updates a value from the [SyncDatabase](https://nsc-de.github.io/js-database/classes/_database_.syncdatabase.html)
 * @see [SyncDatabase](https://nsc-de.github.io/js-database/classes/_database_.syncdatabase.html).[contains()](https://nsc-de.github.io/js-database/classes/_database_.syncdatabase.html#contains) - checks if the [SyncDatabase](https://nsc-de.github.io/js-database/classes/_database_.syncdatabase.html) contains a value
 * @see [SyncDatabase](https://nsc-de.github.io/js-database/classes/_database_.syncdatabase.html).[generateId()](https://nsc-de.github.io/js-database/classes/_database_.syncdatabase.html#contains) - checks if the [SyncDatabase](https://nsc-de.github.io/js-database/classes/_database_.syncdatabase.html) - Generates an id using a namespace
 */
export declare class SyncDatabase extends DatabaseObject {
    adapter: SyncDatabaseAdapter;
    /**
     * Constructor for Database class
     *
     * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
     * @param adapter the adapter to get the data from / save the data to
     * @param data the data to create the database with
     *
     * @see [SyncDatabase](https://nsc-de.github.io/js-database/classes/_database_.syncdatabase.html) - 👩‍👦 the parent class
     */
    constructor(adapter: SyncDatabaseAdapter, data?: JSObject<DatabaseValueAble>);
    /**
     * 💾 Save the data from the database
     *
     * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
     * @returns the database itself, so you can chain operations
     *
     * @see [SyncDatabase](https://nsc-de.github.io/js-database/classes/_database_.syncdatabase.html) - 👩‍👦 the parent class
     */
    saveData(): this;
    /**
     * 🔄 Reload the data from the database
     *
     * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
     * @returns the database itself, so you can chain operations
     *
     * @see [SyncDatabase](https://nsc-de.github.io/js-database/classes/_database_.syncdatabase.html) - 👩‍👦 the parent class
     */
    reloadData(): this;
}
/**
 * Creates a database from a given adapter
 *
 * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
 * @param adapter the databaseAdapter
 * @returns ⌛ Promise (Ready when loaded) >> the database
 */
export declare function createDatabase(adapter: DatabaseAdapter): Promise<Database>;
/**
 * Creates a sync-database from a given adapter
 *
 * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
 * @param adapter the databaseAdapter
 * @returns the SyncDatabase
 */
export declare function createDatabase(adapter: SyncDatabaseAdapter): SyncDatabase;
/**
 * Creates a database from a given adapter
 *
 * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
 * @param adapter the databaseAdapter
 * @returns the Database
 */
export declare function createDatabase(adapter: DatabaseAdapter | SyncDatabaseAdapter): Promise<Database> | SyncDatabase;
/**
 * A value as it is stored in the Database
 *
 * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
 */
export declare type DatabaseValue = DatabaseObject | DatabaseArray | number | string | boolean | null | undefined;
/**
 * The Primitives of the [DatabaseValue](https://nsc-de.github.io/js-database/modules/_database_.html#databasevalue)
 *
 * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
 */
export declare type DatabaseValueAble = DatabaseValueAble[] | JSObject<DatabaseValueAble> | number | string | boolean | null | undefined;
/**
 * Everything that can be inserted into a database ([DatabaseValue](https://nsc-de.github.io/js-database/modules/_database_.html#databasevalue)
 * or [DatabaseValueAble](https://nsc-de.github.io/js-database/modules/_database_.html#databasevalueable))
 *
 * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
 */
export declare type DatabaseInsertable = DatabaseValue | DatabaseValueAble;
export interface JSObject<T = any> {
    [key: string]: T;
}
