import { DatabaseAdapter, JSObject, SyncDatabaseAdapter } from './database';
/**
 * XMLFileAdapter is an async adapter for working with XML Files
 *
 * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
 */
export declare class XMLFileAdapter implements DatabaseAdapter {
    readonly path: string;
    settings: XMLAdapterSettings;
    /**
     * The constructor for [XMLFileAdapter](https://nsc-de.github.io/js-database/classes/_xml_adapter_.xmlfileadapter.html)
     *
     * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
     * @param path the path to a xml file to save to and load from
     *
     * @see [XMLFileAdapter](https://nsc-de.github.io/js-database/classes/_xml_adapter_.xmlfileadapter.html) - 👩‍👦 the parent class
     */
    constructor(path: string, settings?: XMLAdapterSettingsInput);
    /**
     * Saves the given data to the file the adapter refers to
     *
     * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
     * @param data The data to save
     * @returns ⌛ Promise (Ready when saved) >> ⛔ void
     *
     * @see [XMLFileAdapter](https://nsc-de.github.io/js-database/classes/_xml_adapter_.xmlfileadapter.html) - 👩‍👦 the parent class
     */
    save(data: JSObject): Promise<void>;
    /**
     * Loads data from the file the adapter refers to
     *
     * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
     * @returns ⌛ Promise (Ready when saved) >> Object
     *
     * @see [XMLFileAdapter](https://nsc-de.github.io/js-database/classes/_xml_adapter_.xmlfileadapter.html) - 👩‍👦 the parent class
     */
    load(): Promise<JSObject>;
}
/**
 * [SyncXMLFileAdapter](https://nsc-de.github.io/js-database/classes/_xml_adapter_.syncxmlfileadapter.html) is an sync adapter for working with XML Files
 *
 * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
 */
export declare class SyncXMLFileAdapter implements SyncDatabaseAdapter {
    readonly path: string;
    settings: XMLAdapterSettings;
    /**
     * The constructor for [SyncXMLFileAdapter](https://nsc-de.github.io/js-database/classes/_xml_adapter_.syncxmlfileadapter.html)
     *
     * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
     * @param path the path to a xml file to save to and load from
     *
     * @see [SyncXMLFileAdapter](https://nsc-de.github.io/js-database/classes/_xml_adapter_.syncxmlfileadapter.html) - 👩‍👦 the parent class
     */
    constructor(path: string, settings?: XMLAdapterSettingsInput);
    /**
     * Saves the given data to the file the adapter refers to
     *
     * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
     * @param data The data to save
     * @returns ⌛ Promise (Ready when saved) >> ⛔ void
     *
     * @see [SyncXMLFileAdapter](https://nsc-de.github.io/js-database/classes/_xml_adapter_.syncxmlfileadapter.html) - 👩‍👦 the parent class
     */
    save(data: JSObject): void;
    /**
     * Loads data from the file the adapter refers to
     *
     * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
     * @returns ⌛ Promise (Ready when saved) >> Object
     *
     * @see [SyncXMLFileAdapter](https://nsc-de.github.io/js-database/classes/_xml_adapter_.syncxmlfileadapter.html) - 👩‍👦 the parent class
     */
    load(): JSObject;
}
/**
 * The settings for [XMLFileAdapter](https://nsc-de.github.io/js-database/classes/_xml_adapter_.xmlfileadapter.html) or
 * [SyncXMLFileAdapter](https://nsc-de.github.io/js-database/classes/_xml_adapter_.syncxmlfileadapter.html):
 *   beautify - should the generated xml be beautified (default: true)
 *   beautify_space - the space to indent the xml with (default: 2)
 *
 * This is the input version of the interface. It gets applied all default values when passed to a
 * [XMLFileAdapter](https://nsc-de.github.io/js-database/classes/_xml_adapter_.xmlfileadapter.html) or
 * [SyncXMLFileAdapter](https://nsc-de.github.io/js-database/classes/_xml_adapter_.syncxmlfileadapter.html)
 *
 * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
 *
 * @see [XMLFileAdapter](https://nsc-de.github.io/js-database/classes/_xml_adapter_.xmlfileadapter.html)
 * @see [SyncXMLFileAdapter](https://nsc-de.github.io/js-database/classes/_xml_adapter_.syncxmlfileadapter.html)
 */
export interface XMLAdapterSettingsInput {
    /**
     * Should the generated xml be beautified?
     *
     * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
     *
     * @see [XMLAdapterSettingsInput](https://nsc-de.github.io/js-database/interfaces/_xml_adapter_.xmladaptersettingsinput.html) - 👩‍👦 the parent interface
     */
    beautify?: boolean;
    /**
     * How many spaces should be used to indent the generated xml?
     *
     * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
     *
     * @see [XMLAdapterSettingsInput](https://nsc-de.github.io/js-database/interfaces/_xml_adapter_.xmladaptersettingsinput.html) - 👩‍👦 the parent interface
     */
    beautify_space?: number;
    /**
     * Should empty tags be generated as `<element></element>` instead of `<element/>`?
     *
     * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
     *
     * @see [XMLAdapterSettingsInput](https://nsc-de.github.io/js-database/interfaces/_xml_adapter_.xmladaptersettingsinput.html) - 👩‍👦 the parent interface
     */
    fullTagEmptyElement?: boolean;
    /**
     * Should attributes be split over multiple lines and indented when necessary?
     *
     * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
     *
     * @see [XMLAdapterSettingsInput](https://nsc-de.github.io/js-database/interfaces/_xml_adapter_.xmladaptersettingsinput.html) - 👩‍👦 the parent interface
     */
    indentAttributes?: boolean;
}
/**
 * The settings for [XMLFileAdapter](https://nsc-de.github.io/js-database/classes/_xml_adapter_.xmlfileadapter.html) or
 * [SyncXMLFileAdapter](https://nsc-de.github.io/js-database/classes/_xml_adapter_.syncxmlfileadapter.html):
 *   beautify - should the generated xml be beautified (default: true)
 *   beautify_space - the space to indent the xml with (default: 2)
 *
 * This is the input version of the interface. It gets applied all default values when passed to a
 * [XMLFileAdapter](https://nsc-de.github.io/js-database/classes/_xml_adapter_.xmlfileadapter.html) or
 * [SyncXMLFileAdapter](https://nsc-de.github.io/js-database/classes/_xml_adapter_.syncxmlfileadapter.html)
 *
 * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
 *
 * @see [XMLFileAdapter](https://nsc-de.github.io/js-database/classes/_xml_adapter_.xmlfileadapter.html)
 * @see [SyncXMLFileAdapter](https://nsc-de.github.io/js-database/classes/_xml_adapter_.syncxmlfileadapter.html)
 */
export interface XMLAdapterSettings {
    /**
     * Should the generated xml be beautified?
     *
     * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
     *
     * @see [XMLAdapterSettingsInput](https://nsc-de.github.io/js-database/interfaces/_xml_adapter_.xmladaptersettingsinput.html) - 👩‍👦 the parent interface
     */
    beautify: boolean;
    /**
     * How many spaces should be used to indent the generated xml?
     *
     * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
     *
     * @see [XMLAdapterSettingsInput](https://nsc-de.github.io/js-database/interfaces/_xml_adapter_.xmladaptersettingsinput.html) - 👩‍👦 the parent interface
     */
    beautify_space: number;
    /**
     * Should empty tags be generated as `<element></element>` instead of `<element/>`?
     *
     * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
     *
     * @see [XMLAdapterSettingsInput](https://nsc-de.github.io/js-database/interfaces/_xml_adapter_.xmladaptersettingsinput.html) - 👩‍👦 the parent interface
     */
    fullTagEmptyElement: boolean;
    /**
     * Should attributes be split over multiple lines and indented when necessary?
     *
     * @author Nicolas Schmidt <[@nsc-de](https://github.com/nsc-de)>
     *
     * @see [XMLAdapterSettingsInput](https://nsc-de.github.io/js-database/interfaces/_xml_adapter_.xmladaptersettingsinput.html) - 👩‍👦 the parent interface
     */
    indentAttributes: boolean;
}
